using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml;

namespace Glow
{
	public class TilemakerForm : Form
	{
		private Image background;
		private Bitmap bitmap;
		private int seed;
		private Random rand;
		// Notwendig, weil sich ValueChanged-Ereignisse und Tastendrücke überschneiden können.
		private bool shouldRecalculate = true;
		// Kulturunabhängiges (invariantes) Zahlenformat zum Speichern verwenden.
		private System.Globalization.NumberFormatInfo format =
			System.Globalization.CultureInfo.InvariantCulture.NumberFormat;

		/// <summary>
		/// Erforderliche Designervariable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.NumericUpDown numericUpDownRoot;
		private System.Windows.Forms.NumericUpDown numericUpDownSlope;
		private System.Windows.Forms.NumericUpDown numericUpDownSize;
		private System.Windows.Forms.NumericUpDown numericUpDownShadow;
		private System.Windows.Forms.NumericUpDown numericUpDownBlue;
		private System.Windows.Forms.NumericUpDown numericUpDownGreen;
		private System.Windows.Forms.NumericUpDown numericUpDownRed;
		private System.Windows.Forms.NumericUpDown numericUpDownBrightness;
		private System.Windows.Forms.NumericUpDown numericUpDownSeed;
		private System.Windows.Forms.Label labelRed;
		private System.Windows.Forms.Label labelShadow;
		private System.Windows.Forms.NumericUpDown numericUpDownFold;
		private System.Windows.Forms.Label labelSeed;
		private System.Windows.Forms.Label labelSize;
		private System.Windows.Forms.Label labelDensity;
		private System.Windows.Forms.NumericUpDown numericUpDownDensity;
		private System.Windows.Forms.NumericUpDown numericUpDownPen1;
		private System.Windows.Forms.Label labelPen1;
		private System.Windows.Forms.NumericUpDown numericUpDownPen2;
		private System.Windows.Forms.NumericUpDown numericUpDownBundling;
		private System.Windows.Forms.Label labelBundling;
		private System.Windows.Forms.Panel panelSettings;
		private System.Windows.Forms.Label labelGreen;
		private System.Windows.Forms.Label labelRoot;
		private System.Windows.Forms.Label labelBrigthness;
		private System.Windows.Forms.Label labelBlue;
		private System.Windows.Forms.Label labelFold;
		private System.Windows.Forms.Label labelSlope;
		private System.Windows.Forms.Label labelPen2;
		private System.Windows.Forms.Label labelHeight;
		private System.Windows.Forms.NumericUpDown numericUpDownHeight;
		private System.Windows.Forms.Label labelWidth;
		private System.Windows.Forms.NumericUpDown numericUpDownWidth;
		private System.Windows.Forms.ComboBox comboBoxType;
		private System.Windows.Forms.Button buttonRandomize;
		private System.Windows.Forms.Button buttonColor;
		private System.Windows.Forms.Button buttonLoad;
		private System.Windows.Forms.Button buttonSave;
		private System.Windows.Forms.ColorDialog colorDialog;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private System.Windows.Forms.SaveFileDialog saveFileDialog;

		public TilemakerForm()
		{
			InitializeComponent();

			// Double-Buffering verzögert das Zeichnen ein wenig, dafür flackert es nicht.
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer,         true);
			SetStyle(ControlStyles.UserPaint,            true);

			string backgroundFilename = "Background.png";
			if (System.IO.File.Exists(backgroundFilename))
				background = Bitmap.FromFile(backgroundFilename);

			// Gibt es eine Möglichkeit, DropDownLists schon im Designer vorzubelegen?
			if (comboBoxType.SelectedIndex < 0)
				comboBoxType.SelectedIndex = 0;

			// Nicht sehr elegant, aber diese Variante ist kompatibel zum Compact Framework.
			openFileDialog.InitialDirectory = System.IO.Path.GetDirectoryName(
				System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).
				Replace(@"file:\", "");
			saveFileDialog.InitialDirectory = openFileDialog.InitialDirectory;
		}

		/// <summary>
		/// Die verwendeten Ressourcen bereinigen.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode für die Designerunterstützung.
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor geändert werden.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TilemakerForm));
			this.buttonRandomize = new System.Windows.Forms.Button();
			this.numericUpDownRoot = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownSlope = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownSize = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownShadow = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownBlue = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownGreen = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownRed = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownBrightness = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownSeed = new System.Windows.Forms.NumericUpDown();
			this.labelRed = new System.Windows.Forms.Label();
			this.labelShadow = new System.Windows.Forms.Label();
			this.numericUpDownFold = new System.Windows.Forms.NumericUpDown();
			this.labelSeed = new System.Windows.Forms.Label();
			this.labelSize = new System.Windows.Forms.Label();
			this.labelDensity = new System.Windows.Forms.Label();
			this.numericUpDownDensity = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownPen1 = new System.Windows.Forms.NumericUpDown();
			this.labelPen1 = new System.Windows.Forms.Label();
			this.numericUpDownPen2 = new System.Windows.Forms.NumericUpDown();
			this.buttonSave = new System.Windows.Forms.Button();
			this.saveFileDialog = new System.Windows.Forms.SaveFileDialog();
			this.numericUpDownBundling = new System.Windows.Forms.NumericUpDown();
			this.labelBundling = new System.Windows.Forms.Label();
			this.panelSettings = new System.Windows.Forms.Panel();
			this.buttonColor = new System.Windows.Forms.Button();
			this.buttonLoad = new System.Windows.Forms.Button();
			this.comboBoxType = new System.Windows.Forms.ComboBox();
			this.numericUpDownHeight = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownWidth = new System.Windows.Forms.NumericUpDown();
			this.labelHeight = new System.Windows.Forms.Label();
			this.labelWidth = new System.Windows.Forms.Label();
			this.labelPen2 = new System.Windows.Forms.Label();
			this.labelFold = new System.Windows.Forms.Label();
			this.labelSlope = new System.Windows.Forms.Label();
			this.labelRoot = new System.Windows.Forms.Label();
			this.labelBrigthness = new System.Windows.Forms.Label();
			this.labelBlue = new System.Windows.Forms.Label();
			this.labelGreen = new System.Windows.Forms.Label();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.colorDialog = new System.Windows.Forms.ColorDialog();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownRoot)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSlope)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownShadow)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBlue)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownGreen)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownRed)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBrightness)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSeed)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownFold)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownDensity)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownPen1)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownPen2)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBundling)).BeginInit();
			this.panelSettings.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownHeight)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownWidth)).BeginInit();
			this.SuspendLayout();
			// 
			// buttonRandomize
			// 
			this.buttonRandomize.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.buttonRandomize.Location = new System.Drawing.Point(12, 89);
			this.buttonRandomize.Name = "buttonRandomize";
			this.buttonRandomize.Size = new System.Drawing.Size(130, 23);
			this.buttonRandomize.TabIndex = 5;
			this.buttonRandomize.Text = "Randomize (F5)";
			this.buttonRandomize.Click += new System.EventHandler(this.buttonRandomize_Click);
			// 
			// numericUpDownRoot
			// 
			this.numericUpDownRoot.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownRoot.DecimalPlaces = 1;
			this.numericUpDownRoot.Increment = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				65536});
			this.numericUpDownRoot.Location = new System.Drawing.Point(244, 91);
			this.numericUpDownRoot.Name = "numericUpDownRoot";
			this.numericUpDownRoot.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownRoot.TabIndex = 13;
			this.numericUpDownRoot.Value = new System.Decimal(new int[] {
																			3,
																			0,
																			0,
																			0});
			this.numericUpDownRoot.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownSlope
			// 
			this.numericUpDownSlope.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownSlope.DecimalPlaces = 1;
			this.numericUpDownSlope.Increment = new System.Decimal(new int[] {
																				 1,
																				 0,
																				 0,
																				 65536});
			this.numericUpDownSlope.Location = new System.Drawing.Point(388, 39);
			this.numericUpDownSlope.Name = "numericUpDownSlope";
			this.numericUpDownSlope.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownSlope.TabIndex = 17;
			this.numericUpDownSlope.Value = new System.Decimal(new int[] {
																			 8,
																			 0,
																			 0,
																			 0});
			this.numericUpDownSlope.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownSize
			// 
			this.numericUpDownSize.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownSize.Location = new System.Drawing.Point(244, 65);
			this.numericUpDownSize.Maximum = new System.Decimal(new int[] {
																			  1000,
																			  0,
																			  0,
																			  0});
			this.numericUpDownSize.Minimum = new System.Decimal(new int[] {
																			  1000,
																			  0,
																			  0,
																			  -2147483648});
			this.numericUpDownSize.Name = "numericUpDownSize";
			this.numericUpDownSize.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownSize.TabIndex = 11;
			this.numericUpDownSize.Value = new System.Decimal(new int[] {
																			32,
																			0,
																			0,
																			0});
			this.numericUpDownSize.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownShadow
			// 
			this.numericUpDownShadow.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownShadow.DecimalPlaces = 1;
			this.numericUpDownShadow.Increment = new System.Decimal(new int[] {
																				  1,
																				  0,
																				  0,
																				  65536});
			this.numericUpDownShadow.Location = new System.Drawing.Point(730, 65);
			this.numericUpDownShadow.Minimum = new System.Decimal(new int[] {
																				100,
																				0,
																				0,
																				-2147483648});
			this.numericUpDownShadow.Name = "numericUpDownShadow";
			this.numericUpDownShadow.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownShadow.TabIndex = 34;
			this.numericUpDownShadow.Value = new System.Decimal(new int[] {
																			  4,
																			  0,
																			  0,
																			  65536});
			this.numericUpDownShadow.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownBlue
			// 
			this.numericUpDownBlue.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownBlue.Location = new System.Drawing.Point(518, 65);
			this.numericUpDownBlue.Maximum = new System.Decimal(new int[] {
																			  255,
																			  0,
																			  0,
																			  0});
			this.numericUpDownBlue.Name = "numericUpDownBlue";
			this.numericUpDownBlue.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownBlue.TabIndex = 25;
			this.numericUpDownBlue.Value = new System.Decimal(new int[] {
																			100,
																			0,
																			0,
																			0});
			this.numericUpDownBlue.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownGreen
			// 
			this.numericUpDownGreen.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownGreen.Location = new System.Drawing.Point(518, 39);
			this.numericUpDownGreen.Maximum = new System.Decimal(new int[] {
																			   255,
																			   0,
																			   0,
																			   0});
			this.numericUpDownGreen.Name = "numericUpDownGreen";
			this.numericUpDownGreen.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownGreen.TabIndex = 23;
			this.numericUpDownGreen.Value = new System.Decimal(new int[] {
																			 200,
																			 0,
																			 0,
																			 0});
			this.numericUpDownGreen.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownRed
			// 
			this.numericUpDownRed.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownRed.Location = new System.Drawing.Point(518, 13);
			this.numericUpDownRed.Maximum = new System.Decimal(new int[] {
																			 255,
																			 0,
																			 0,
																			 0});
			this.numericUpDownRed.Name = "numericUpDownRed";
			this.numericUpDownRed.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownRed.TabIndex = 21;
			this.numericUpDownRed.Value = new System.Decimal(new int[] {
																		   200,
																		   0,
																		   0,
																		   0});
			this.numericUpDownRed.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownBrightness
			// 
			this.numericUpDownBrightness.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownBrightness.Increment = new System.Decimal(new int[] {
																					  8,
																					  0,
																					  0,
																					  0});
			this.numericUpDownBrightness.Location = new System.Drawing.Point(518, 91);
			this.numericUpDownBrightness.Maximum = new System.Decimal(new int[] {
																					255,
																					0,
																					0,
																					0});
			this.numericUpDownBrightness.Name = "numericUpDownBrightness";
			this.numericUpDownBrightness.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownBrightness.TabIndex = 27;
			this.numericUpDownBrightness.Value = new System.Decimal(new int[] {
																				  64,
																				  0,
																				  0,
																				  0});
			this.numericUpDownBrightness.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownSeed
			// 
			this.numericUpDownSeed.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownSeed.Location = new System.Drawing.Point(388, 13);
			this.numericUpDownSeed.Maximum = new System.Decimal(new int[] {
																			  1000,
																			  0,
																			  0,
																			  0});
			this.numericUpDownSeed.Name = "numericUpDownSeed";
			this.numericUpDownSeed.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownSeed.TabIndex = 15;
			this.numericUpDownSeed.Value = new System.Decimal(new int[] {
																			32,
																			0,
																			0,
																			0});
			this.numericUpDownSeed.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// labelRed
			// 
			this.labelRed.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelRed.Location = new System.Drawing.Point(444, 15);
			this.labelRed.Name = "labelRed";
			this.labelRed.Size = new System.Drawing.Size(74, 16);
			this.labelRed.TabIndex = 20;
			this.labelRed.Text = "&Red:";
			this.labelRed.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labelShadow
			// 
			this.labelShadow.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelShadow.Location = new System.Drawing.Point(574, 67);
			this.labelShadow.Name = "labelShadow";
			this.labelShadow.Size = new System.Drawing.Size(156, 16);
			this.labelShadow.TabIndex = 33;
			this.labelShadow.Text = "Lightning/shadow offset:";
			this.labelShadow.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// numericUpDownFold
			// 
			this.numericUpDownFold.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownFold.Location = new System.Drawing.Point(388, 65);
			this.numericUpDownFold.Name = "numericUpDownFold";
			this.numericUpDownFold.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownFold.TabIndex = 19;
			this.numericUpDownFold.Value = new System.Decimal(new int[] {
																			2,
																			0,
																			0,
																			0});
			this.numericUpDownFold.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// labelSeed
			// 
			this.labelSeed.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelSeed.Location = new System.Drawing.Point(300, 15);
			this.labelSeed.Name = "labelSeed";
			this.labelSeed.Size = new System.Drawing.Size(88, 16);
			this.labelSeed.TabIndex = 14;
			this.labelSeed.Text = "Slope seed:";
			this.labelSeed.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelSize
			// 
			this.labelSize.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelSize.Location = new System.Drawing.Point(148, 67);
			this.labelSize.Name = "labelSize";
			this.labelSize.Size = new System.Drawing.Size(96, 16);
			this.labelSize.TabIndex = 10;
			this.labelSize.Text = "Feature size (+/-):";
			this.labelSize.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelDensity
			// 
			this.labelDensity.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelDensity.Location = new System.Drawing.Point(148, 15);
			this.labelDensity.Name = "labelDensity";
			this.labelDensity.Size = new System.Drawing.Size(96, 16);
			this.labelDensity.TabIndex = 6;
			this.labelDensity.Text = "&Density:";
			this.labelDensity.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// numericUpDownDensity
			// 
			this.numericUpDownDensity.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownDensity.Increment = new System.Decimal(new int[] {
																				   10,
																				   0,
																				   0,
																				   0});
			this.numericUpDownDensity.Location = new System.Drawing.Point(244, 13);
			this.numericUpDownDensity.Maximum = new System.Decimal(new int[] {
																				 10000,
																				 0,
																				 0,
																				 0});
			this.numericUpDownDensity.Name = "numericUpDownDensity";
			this.numericUpDownDensity.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownDensity.TabIndex = 7;
			this.numericUpDownDensity.Value = new System.Decimal(new int[] {
																			   500,
																			   0,
																			   0,
																			   0});
			this.numericUpDownDensity.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownPen1
			// 
			this.numericUpDownPen1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownPen1.DecimalPlaces = 1;
			this.numericUpDownPen1.Increment = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				65536});
			this.numericUpDownPen1.Location = new System.Drawing.Point(730, 13);
			this.numericUpDownPen1.Name = "numericUpDownPen1";
			this.numericUpDownPen1.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownPen1.TabIndex = 30;
			this.numericUpDownPen1.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// labelPen1
			// 
			this.labelPen1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelPen1.Location = new System.Drawing.Point(628, 15);
			this.labelPen1.Name = "labelPen1";
			this.labelPen1.Size = new System.Drawing.Size(102, 16);
			this.labelPen1.TabIndex = 29;
			this.labelPen1.Text = "1st &pen width:";
			this.labelPen1.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// numericUpDownPen2
			// 
			this.numericUpDownPen2.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownPen2.DecimalPlaces = 1;
			this.numericUpDownPen2.Increment = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				65536});
			this.numericUpDownPen2.Location = new System.Drawing.Point(730, 39);
			this.numericUpDownPen2.Name = "numericUpDownPen2";
			this.numericUpDownPen2.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownPen2.TabIndex = 32;
			this.numericUpDownPen2.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// buttonSave
			// 
			this.buttonSave.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonSave.Location = new System.Drawing.Point(680, 89);
			this.buttonSave.Name = "buttonSave";
			this.buttonSave.Size = new System.Drawing.Size(100, 23);
			this.buttonSave.TabIndex = 36;
			this.buttonSave.Text = "&Save as... (F2)";
			this.buttonSave.Click += new System.EventHandler(this.buttonSave_Click);
			// 
			// saveFileDialog
			// 
			this.saveFileDialog.Filter = "Glow Tilemaker settings (*.xml)|*.xml|PNG image (*.png)|*.png";
			this.saveFileDialog.FilterIndex = 2;
			this.saveFileDialog.Title = "Save as...";
			// 
			// numericUpDownBundling
			// 
			this.numericUpDownBundling.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownBundling.Location = new System.Drawing.Point(244, 39);
			this.numericUpDownBundling.Maximum = new System.Decimal(new int[] {
																				  1000,
																				  0,
																				  0,
																				  0});
			this.numericUpDownBundling.Name = "numericUpDownBundling";
			this.numericUpDownBundling.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownBundling.TabIndex = 9;
			this.numericUpDownBundling.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// labelBundling
			// 
			this.labelBundling.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelBundling.Location = new System.Drawing.Point(148, 41);
			this.labelBundling.Name = "labelBundling";
			this.labelBundling.Size = new System.Drawing.Size(96, 16);
			this.labelBundling.TabIndex = 8;
			this.labelBundling.Text = "Bundling:";
			this.labelBundling.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// panelSettings
			// 
			this.panelSettings.Controls.Add(this.buttonColor);
			this.panelSettings.Controls.Add(this.buttonLoad);
			this.panelSettings.Controls.Add(this.comboBoxType);
			this.panelSettings.Controls.Add(this.numericUpDownHeight);
			this.panelSettings.Controls.Add(this.numericUpDownWidth);
			this.panelSettings.Controls.Add(this.buttonSave);
			this.panelSettings.Controls.Add(this.buttonRandomize);
			this.panelSettings.Controls.Add(this.numericUpDownBundling);
			this.panelSettings.Controls.Add(this.numericUpDownRoot);
			this.panelSettings.Controls.Add(this.numericUpDownSlope);
			this.panelSettings.Controls.Add(this.numericUpDownPen2);
			this.panelSettings.Controls.Add(this.numericUpDownSize);
			this.panelSettings.Controls.Add(this.numericUpDownShadow);
			this.panelSettings.Controls.Add(this.numericUpDownPen1);
			this.panelSettings.Controls.Add(this.numericUpDownBlue);
			this.panelSettings.Controls.Add(this.numericUpDownGreen);
			this.panelSettings.Controls.Add(this.numericUpDownDensity);
			this.panelSettings.Controls.Add(this.numericUpDownRed);
			this.panelSettings.Controls.Add(this.numericUpDownBrightness);
			this.panelSettings.Controls.Add(this.numericUpDownSeed);
			this.panelSettings.Controls.Add(this.numericUpDownFold);
			this.panelSettings.Controls.Add(this.labelHeight);
			this.panelSettings.Controls.Add(this.labelWidth);
			this.panelSettings.Controls.Add(this.labelPen2);
			this.panelSettings.Controls.Add(this.labelFold);
			this.panelSettings.Controls.Add(this.labelSlope);
			this.panelSettings.Controls.Add(this.labelRoot);
			this.panelSettings.Controls.Add(this.labelBrigthness);
			this.panelSettings.Controls.Add(this.labelBlue);
			this.panelSettings.Controls.Add(this.labelGreen);
			this.panelSettings.Controls.Add(this.labelBundling);
			this.panelSettings.Controls.Add(this.labelPen1);
			this.panelSettings.Controls.Add(this.labelDensity);
			this.panelSettings.Controls.Add(this.labelSize);
			this.panelSettings.Controls.Add(this.labelSeed);
			this.panelSettings.Controls.Add(this.labelRed);
			this.panelSettings.Controls.Add(this.labelShadow);
			this.panelSettings.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panelSettings.Location = new System.Drawing.Point(0, 368);
			this.panelSettings.Name = "panelSettings";
			this.panelSettings.Size = new System.Drawing.Size(792, 123);
			this.panelSettings.TabIndex = 0;
			// 
			// buttonColor
			// 
			this.buttonColor.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonColor.Location = new System.Drawing.Point(574, 11);
			this.buttonColor.Name = "buttonColor";
			this.buttonColor.Size = new System.Drawing.Size(48, 23);
			this.buttonColor.TabIndex = 28;
			this.buttonColor.Text = "&Color";
			this.buttonColor.Click += new System.EventHandler(this.buttonColor_Click);
			// 
			// buttonLoad
			// 
			this.buttonLoad.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonLoad.Location = new System.Drawing.Point(574, 89);
			this.buttonLoad.Name = "buttonLoad";
			this.buttonLoad.Size = new System.Drawing.Size(100, 23);
			this.buttonLoad.TabIndex = 35;
			this.buttonLoad.Text = "&Load... (F3)";
			this.buttonLoad.Click += new System.EventHandler(this.buttonLoad_Click);
			// 
			// comboBoxType
			// 
			this.comboBoxType.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.comboBoxType.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboBoxType.Items.AddRange(new object[] {
															  "Grass and roots",
															  "Debris",
															  "Bricks",
															  "Edges/jags"});
			this.comboBoxType.Location = new System.Drawing.Point(12, 12);
			this.comboBoxType.Name = "comboBoxType";
			this.comboBoxType.Size = new System.Drawing.Size(130, 21);
			this.comboBoxType.TabIndex = 0;
			this.comboBoxType.SelectedIndexChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownHeight
			// 
			this.numericUpDownHeight.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownHeight.Increment = new System.Decimal(new int[] {
																				  2,
																				  0,
																				  0,
																				  0});
			this.numericUpDownHeight.Location = new System.Drawing.Point(92, 65);
			this.numericUpDownHeight.Maximum = new System.Decimal(new int[] {
																				1024,
																				0,
																				0,
																				0});
			this.numericUpDownHeight.Minimum = new System.Decimal(new int[] {
																				2,
																				0,
																				0,
																				0});
			this.numericUpDownHeight.Name = "numericUpDownHeight";
			this.numericUpDownHeight.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownHeight.TabIndex = 4;
			this.numericUpDownHeight.Value = new System.Decimal(new int[] {
																			  32,
																			  0,
																			  0,
																			  0});
			this.numericUpDownHeight.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// numericUpDownWidth
			// 
			this.numericUpDownWidth.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.numericUpDownWidth.Increment = new System.Decimal(new int[] {
																				 32,
																				 0,
																				 0,
																				 0});
			this.numericUpDownWidth.Location = new System.Drawing.Point(92, 39);
			this.numericUpDownWidth.Maximum = new System.Decimal(new int[] {
																			   1024,
																			   0,
																			   0,
																			   0});
			this.numericUpDownWidth.Minimum = new System.Decimal(new int[] {
																			   32,
																			   0,
																			   0,
																			   0});
			this.numericUpDownWidth.Name = "numericUpDownWidth";
			this.numericUpDownWidth.Size = new System.Drawing.Size(50, 20);
			this.numericUpDownWidth.TabIndex = 2;
			this.numericUpDownWidth.Value = new System.Decimal(new int[] {
																			 512,
																			 0,
																			 0,
																			 0});
			this.numericUpDownWidth.ValueChanged += new System.EventHandler(this.numericUpDown_ValueChanged);
			// 
			// labelHeight
			// 
			this.labelHeight.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelHeight.Location = new System.Drawing.Point(12, 67);
			this.labelHeight.Name = "labelHeight";
			this.labelHeight.Size = new System.Drawing.Size(80, 16);
			this.labelHeight.TabIndex = 3;
			this.labelHeight.Text = "Bitmap &height:";
			this.labelHeight.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelWidth
			// 
			this.labelWidth.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelWidth.Location = new System.Drawing.Point(12, 41);
			this.labelWidth.Name = "labelWidth";
			this.labelWidth.Size = new System.Drawing.Size(80, 16);
			this.labelWidth.TabIndex = 1;
			this.labelWidth.Text = "Bitmap &width:";
			this.labelWidth.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelPen2
			// 
			this.labelPen2.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelPen2.Location = new System.Drawing.Point(574, 41);
			this.labelPen2.Name = "labelPen2";
			this.labelPen2.Size = new System.Drawing.Size(156, 16);
			this.labelPen2.TabIndex = 31;
			this.labelPen2.Text = "2nd pen width:";
			this.labelPen2.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelFold
			// 
			this.labelFold.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelFold.Location = new System.Drawing.Point(300, 67);
			this.labelFold.Name = "labelFold";
			this.labelFold.Size = new System.Drawing.Size(88, 16);
			this.labelFold.TabIndex = 18;
			this.labelFold.Text = "Maximum &fold:";
			this.labelFold.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelSlope
			// 
			this.labelSlope.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelSlope.Location = new System.Drawing.Point(300, 41);
			this.labelSlope.Name = "labelSlope";
			this.labelSlope.Size = new System.Drawing.Size(88, 16);
			this.labelSlope.TabIndex = 16;
			this.labelSlope.Text = "Maximum slope:";
			this.labelSlope.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelRoot
			// 
			this.labelRoot.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelRoot.Location = new System.Drawing.Point(148, 93);
			this.labelRoot.Name = "labelRoot";
			this.labelRoot.Size = new System.Drawing.Size(96, 16);
			this.labelRoot.TabIndex = 12;
			this.labelRoot.Text = "Feature root:";
			this.labelRoot.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelBrigthness
			// 
			this.labelBrigthness.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelBrigthness.Location = new System.Drawing.Point(300, 93);
			this.labelBrigthness.Name = "labelBrigthness";
			this.labelBrigthness.Size = new System.Drawing.Size(218, 16);
			this.labelBrigthness.TabIndex = 26;
			this.labelBrigthness.Text = "Brightness distribution:";
			this.labelBrigthness.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labelBlue
			// 
			this.labelBlue.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelBlue.Location = new System.Drawing.Point(444, 67);
			this.labelBlue.Name = "labelBlue";
			this.labelBlue.Size = new System.Drawing.Size(74, 16);
			this.labelBlue.TabIndex = 24;
			this.labelBlue.Text = "&Blue:";
			this.labelBlue.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// labelGreen
			// 
			this.labelGreen.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.labelGreen.Location = new System.Drawing.Point(444, 41);
			this.labelGreen.Name = "labelGreen";
			this.labelGreen.Size = new System.Drawing.Size(74, 16);
			this.labelGreen.TabIndex = 22;
			this.labelGreen.Text = "&Green:";
			this.labelGreen.TextAlign = System.Drawing.ContentAlignment.MiddleRight;
			// 
			// openFileDialog
			// 
			this.openFileDialog.Filter = "Glow Tilemaker settings (*.xml)|*.xml";
			this.openFileDialog.Title = "Open...";
			// 
			// colorDialog
			// 
			this.colorDialog.Color = System.Drawing.Color.FromArgb(((System.Byte)(200)), ((System.Byte)(200)), ((System.Byte)(100)));
			this.colorDialog.FullOpen = true;
			// 
			// TilemakerForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(792, 491);
			this.Controls.Add(this.panelSettings);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.KeyPreview = true;
			this.MinimumSize = new System.Drawing.Size(800, 157);
			this.Name = "TilemakerForm";
			this.Text = "Glow Tilemaker 0.3.1 alpha";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.TilemakerForm_KeyDown);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.TilemakerForm_Paint);
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownRoot)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSlope)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownShadow)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBlue)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownGreen)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownRed)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBrightness)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSeed)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownFold)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownDensity)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownPen1)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownPen2)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBundling)).EndInit();
			this.panelSettings.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownHeight)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownWidth)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Der Haupteinstiegspunkt für die Anwendung.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.EnableVisualStyles();
			Application.Run(new TilemakerForm());
		}

		private void TilemakerForm_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.F2:
					buttonSave_Click(sender, e);
					break;
				case Keys.F3:
					buttonLoad_Click(sender, e);
					break;
				case Keys.F5:
					// Abbruch, es wird sowieso gerade neu berechnet. Wichtig, sonst
					// würden sich die beiden Funktionsaufrufe gegenseitig stören.
					if (!shouldRecalculate)
						buttonRandomize_Click(sender, e);
					break;
				case Keys.F6:
					Random r = new Random();
					foreach (Control control in panelSettings.Controls)
					{
						if (control is NumericUpDown && control != numericUpDownWidth && control != numericUpDownHeight)
						{
							NumericUpDown c = control as NumericUpDown;
							Decimal range = (c.Maximum - c.Minimum) / 100;
							Decimal v = c.Value + (decimal)r.Next() / (decimal)int.MaxValue * range - range / 2;
							c.Value = Math.Min(Math.Max(c.Minimum, v), c.Maximum);
						}
					}
					break;
				case Keys.F7:
					if (ActiveControl is NumericUpDown)
					{
						NumericUpDown c = ActiveControl as NumericUpDown;
						c.Value = c.Minimum + (decimal)new Random().Next() / (decimal)int.MaxValue * (c.Maximum - c.Minimum);
					}
					break;
			}
		}

		private void TilemakerForm_Paint(object sender, PaintEventArgs e)
		{
			int y = 0, scale = 4, padding = 16;

			if (shouldRecalculate)
				CreateBitmap();

			Graphics g = e.Graphics;
			g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;

			// Paint to screen.
			g.Clear(Color.Black);
			g.DrawImageUnscaled(bitmap, 0, y);
			y += bitmap.Height + padding;
			g.DrawImage(bitmap, 0, y, bitmap.Width * scale, bitmap.Height * scale);
			y += bitmap.Height * scale + padding;
			if (y >= panelSettings.Top)
				return;

			// Für die Beurteilung der Transparenz auf schwarzem/grauem Grund zeichnen.
			if (background != null)
				g.DrawImageUnscaled(background, 0, y);
			else
				g.FillRectangle(Brushes.Gray, 0, y, Width, bitmap.Height * (1 + scale) + padding);
			g.DrawImageUnscaled(bitmap, 0, y);
			y += bitmap.Height + padding;
			if (y >= panelSettings.Top)
				return;
			g.DrawImage(bitmap, 0, y, bitmap.Width * scale, bitmap.Height * scale);
		}

		private void numericUpDown_ValueChanged(object sender, EventArgs e)
		{
			shouldRecalculate = true;
			Invalidate();
		}

		private void buttonRandomize_Click(object sender, System.EventArgs e)
		{
			if (rand != null)
				seed = rand.Next();
			else
				seed = new Random().Next();
			numericUpDown_ValueChanged(sender, e);
		}

		private void buttonColor_Click(object sender, System.EventArgs e)
		{
			colorDialog.Color = Color.FromArgb(
				(int)numericUpDownRed.Value,
				(int)numericUpDownGreen.Value,
				(int)numericUpDownBlue.Value);
			colorDialog.ShowDialog();
			numericUpDownRed.Value   = colorDialog.Color.R;
			numericUpDownGreen.Value = colorDialog.Color.G;
			numericUpDownBlue.Value  = colorDialog.Color.B;
		}

		private void buttonLoad_Click(object sender, EventArgs e)
		{
			if (openFileDialog.ShowDialog() != DialogResult.OK)
				return;

			if (!LoadSettings(openFileDialog.FileName))
			{
				MessageBox.Show("Not a valid settings file.",
					"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			saveFileDialog.FileName = openFileDialog.FileName;
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			if (bitmap == null)
				return;

			// Datenendung verwerfen, damit man problemlos das Dateiformat wechseln kann.
			saveFileDialog.FileName = new Regex(@"\.[^.]*$",
				RegexOptions.IgnoreCase).Replace(saveFileDialog.FileName, "");
			if (saveFileDialog.ShowDialog() != DialogResult.OK)
				return;
			string filename = saveFileDialog.FileName;

			// Die eingegebene Dateiendung wird praktisch ignoriert, relevant ist das im
			// Filter ausgewählte Dateiformat.
			string[] filter = saveFileDialog.Filter.Split("|".ToCharArray());
			int index = saveFileDialog.FilterIndex * 2 - 1;
			if (index >= filter.Length)
			{
				MessageBox.Show("Invalid file format.", "Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				return;
			}

			switch (filter[index].Substring(2, 3).ToLower())
			{
				case "bmp":
					if (!filename.ToLower().EndsWith(".bmp"))
						filename += ".bmp";
					bitmap.Save(filename, ImageFormat.Bmp);
					break;
				case "gif":
					if (!filename.ToLower().EndsWith(".gif"))
						filename += ".gif";
					bitmap.Save(filename, ImageFormat.Gif);
					break;
				case "png":
					if (!filename.ToLower().EndsWith(".png"))
						filename += ".png";
					bitmap.Save(filename, ImageFormat.Png);
					break;
				case "tif":
					if (!filename.ToLower().EndsWith(".tif") &&
						!filename.ToLower().EndsWith(".tiff"))
						filename += ".tif";
					bitmap.Save(filename, ImageFormat.Tiff);
					break;
				default:
					if (!filename.ToLower().EndsWith(".xml"))
						filename += ".xml";
					SaveSettings(filename);
					openFileDialog.FileName = filename;
					break;
			}
		}

		private bool LoadSettings(string filename)
		{
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load(filename);

				XmlNodeList window = doc.GetElementsByTagName("Window");
				if (window.Count > 0)
				{
					Width  = Convert.ToInt32(window[0].Attributes["width"].Value);
					Height = Convert.ToInt32(window[0].Attributes["height"].Value);
				}

				foreach (Control control in panelSettings.Controls)
				{
					XmlNode node = doc.SelectSingleNode("//Setting[@name='" + control.Name + "']/@value");
					if (node == null || node.NodeType != XmlNodeType.Attribute)
						continue;
					if (control is NumericUpDown)
						(control as NumericUpDown).Value = Convert.ToDecimal(node.Value, format);
					else if (control is ComboBox)
						(control as ComboBox).SelectedIndex = Convert.ToInt32(node.Value, format);
				}
			}
			catch
			{
				return false;
			}
			return true;
		}

		private void SaveSettings(string filename)
		{
			XmlDocument doc        = new XmlDocument();
			XmlNode root           = doc.CreateElement("TilemakerSettings");

			XmlNode node           = doc.CreateElement("Window");
			XmlAttribute attribute = doc.CreateAttribute("width");
			attribute.Value        = Width.ToString(format);
			node.Attributes.Append(attribute);
			attribute              = doc.CreateAttribute("height");
			attribute.Value        = Height.ToString(format);
			node.Attributes.Append(attribute);
			root.AppendChild(node);

			foreach (Control control in panelSettings.Controls)
			{
				string value;
				if (control is ComboBox)
					value = (control as ComboBox).SelectedIndex.ToString(format);
				else if (control is NumericUpDown)
					value = (control as NumericUpDown).Value.ToString(format);
				else
					continue;
				node            = doc.CreateElement("Setting");
				attribute       = doc.CreateAttribute("name");
				attribute.Value = control.Name;
				node.Attributes.Append(attribute);
				attribute       = doc.CreateAttribute("value");
				attribute.Value = value;
				node.Attributes.Append(attribute);
				root.AppendChild(node);
			}
			doc.AppendChild(root);
			doc.Save(filename);
		}

		private Color GetColor(bool randomize)
		{
			int brightness = 0;
			if (randomize)
			{
				brightness = rand.Next(
					(int)numericUpDownBrightness.Value * 2) -
					(int)numericUpDownBrightness.Value;
			}
			return Color.FromArgb(
				Math.Max(0, Math.Min(255, (int)numericUpDownRed.Value   + brightness)),
				Math.Max(0, Math.Min(255, (int)numericUpDownGreen.Value + brightness)),
				Math.Max(0, Math.Min(255, (int)numericUpDownBlue.Value  + brightness)));
		}

		private void CreateBitmap()
		{
			rand = new Random(seed);
			bitmap = new Bitmap((int)numericUpDownWidth.Value, (int)numericUpDownHeight.Value);
			Graphics g = Graphics.FromImage(bitmap);
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			//g.Clear(Color.Transparent);

			switch (comboBoxType.SelectedIndex)
			{
				case 1:
					CreateDebris(g);
					break;
				case 2:
					CreateBricks(g);
					break;
				case 3:
					CreateEdgesOrJags(g);
					break;
				default:
					CreateGrassAndRoots(g);
					break;
			}

			shouldRecalculate = false;
		}

		private void CreateGrassAndRoots(Graphics g)
		{
			Pen penThin  = new Pen(Color.White, (float)Math.Min(numericUpDownPen1.Value, numericUpDownPen2.Value));
			Pen penThick = new Pen(Color.White, (float)Math.Max(numericUpDownPen1.Value, numericUpDownPen2.Value));

			int density        = (int)numericUpDownDensity.Value;
			int bundling       = (int)numericUpDownBundling.Value, bundlingCurrent = 0;
			double root        = (double)numericUpDownRoot.Value;
			float x            = 0;
			float featureSize  = (float)Math.Abs(numericUpDownSize.Value);
			float slopeSeed    = (float)numericUpDownSeed.Value;
			float maximumSlope = (float)numericUpDownSlope.Value;
			float fold         = (float)numericUpDownFold.Value;
			float shadowDelta  = (float)numericUpDownShadow.Value;

			for (int i = 0; i < density; i++)
			{
				double maxValue = Math.Pow(featureSize, root);
				float h = featureSize - (float)Math.Pow((double)rand.Next() / (double)int.MaxValue * maxValue, 1.0 / root);
				float d = (float)rand.Next() / (float)int.MaxValue * (2.0f * slopeSeed) - slopeSeed;
				if (h > 1)
					d /= h;
				if (d > maximumSlope)
					d = maximumSlope;
				if (d < -maximumSlope)
					d = -maximumSlope;

				float y0 = 0, y1 = h - 1;
				if (numericUpDownSize.Value >= 0)
				{
					y1 = bitmap.Height - 1;
					y0 = y1 - h + 1;
				}

				penThin.Color = GetColor(true);

				if (--bundlingCurrent <= 0)
				{
					x = (float)rand.Next() / (float)int.MaxValue * (float)bitmap.Width;
					bundlingCurrent = rand.Next(bundling + 1);
				}

				float foldDelta = (float)rand.Next() / (float)int.MaxValue * fold - (fold * 0.5f);
				float foldX = x + (d * 0.5f) + foldDelta;
				float foldY = (y0 - y1) * ((float)rand.Next() / (float)int.MaxValue) + y1;

				PointF[] points = new PointF[3] {
													new PointF(x + d + shadowDelta, y0),
													new PointF(foldX + shadowDelta, foldY),
													new PointF(x + shadowDelta, y1),
				};
				g.DrawLines(Pens.Black, points);
				if (penThick.Width > penThin.Width)
				{
					penThick.Color = Color.Black;
					if (numericUpDownPen1.Value > numericUpDownPen2.Value)
						g.DrawLine(penThick, points[1], points[2]);
					else
						g.DrawLine(penThick, points[0], points[1]);
				}

				points = new PointF[3] {
										   new PointF(x + d, y0),
										   new PointF(foldX, foldY),
										   new PointF(x, y1),
				};
				g.DrawLines(penThin, points);
				if (penThick.Width > penThin.Width)
				{
					penThick.Color = penThin.Color;
					if (numericUpDownPen1.Value > numericUpDownPen2.Value)
						g.DrawLine(penThick, points[1], points[2]);
					else
						g.DrawLine(penThick, points[0], points[1]);
				}
			}
		}

		private void CreateDebris(Graphics g)
		{
			int density       = (int)numericUpDownDensity.Value;
			double root       = (double)numericUpDownRoot.Value;
			float featureSize = (float)Math.Abs(numericUpDownSize.Value);
			float shadowDelta = (float)numericUpDownShadow.Value;
			float slope       = Math.Min(1.0f, (float)numericUpDownSlope.Value) - 0.5f;

			for (int i = 0; i < density; i++)
			{
				int maxValue = (int)Math.Round(Math.Pow(featureSize, root));
				if (maxValue < 0)
					continue;
				float h = featureSize - (float)Math.Pow(rand.Next(maxValue), 1.0 / root);

				float x = (float)rand.Next() / (float)int.MaxValue * (float)bitmap.Width;
				//x += h * 0.5f;
				float y = h * slope;
				if (numericUpDownSize.Value >= 0)
				{
					y = bitmap.Height - 1 - h * slope;
				}

				Color color = GetColor(true);

				float angle = rand.Next(360);

				g.TranslateTransform(x + shadowDelta, y);
				g.RotateTransform(angle);
				g.TranslateTransform(-x, -y);
				//g.FillEllipse(Brushes.Black, x + shadowDelta, y, h, h);
				g.FillRectangle(Brushes.Black, x - h * 0.5f, y - h * 0.5f, h, h);
				g.ResetTransform();

				g.TranslateTransform(x, y);
				g.RotateTransform(angle);
				g.TranslateTransform(-x, -y);
				//g.FillEllipse(new SolidBrush(color), x, y, h, h);
				g.FillRectangle(new SolidBrush(color), x - h * 0.5f, y - h * 0.5f, h, h);
				g.ResetTransform();

				float highlightOffset = 0.4f, highlightSize = 0.4f;
				int brightnessHighlight = (int)numericUpDownBrightness.Value;
				Color colorHighlight = Color.FromArgb(
					Math.Min(255, color.R + brightnessHighlight),
					Math.Min(255, color.G + brightnessHighlight),
					Math.Min(255, color.B + brightnessHighlight));
				g.FillEllipse(new SolidBrush(colorHighlight),
					x - h * highlightOffset, y - h * highlightOffset,
					h * highlightSize, h * highlightSize);
			}
		}

		private void CreateBricks(Graphics g)
		{
			Pen penHighlight  = new Pen(Color.White, (float)numericUpDownPen1.Value);
			Pen penShadow     = new Pen(Color.Black, (float)numericUpDownPen1.Value);
			Pen penScratch    = new Pen(Color.Black, (float)numericUpDownPen2.Value);

			int density       = (int)numericUpDownDensity.Value;
			int height        = Math.Max(2, (int)Math.Abs(numericUpDownSize.Value));
			int width         = 2 * height;
			int bundling      = (int)numericUpDownBundling.Value % width;
			float space       = (float)numericUpDownRoot.Value;
			float slope       = (float)numericUpDownSlope.Value;
			float shadowDelta = (float)numericUpDownShadow.Value;

			g.Clear(Color.Black);

			for (int y = 0; y < bitmap.Height - 1; y += height)
			{
				int x = y % (2 * height) == 0 ? 0 : -width + bundling;
				for (; x < bitmap.Width - 1; x += width)
				{
					Color color = GetColor(true);
					int brightnessHighlight = (int)numericUpDownBrightness.Value;
					penHighlight.Color = Color.FromArgb(
						Math.Min(255, color.R + brightnessHighlight),
						Math.Min(255, color.G + brightnessHighlight),
						Math.Min(255, color.B + brightnessHighlight));
					penShadow.Color = Color.FromArgb(
						Math.Max(0, color.R - brightnessHighlight),
						Math.Max(0, color.G - brightnessHighlight),
						Math.Max(0, color.B - brightnessHighlight));
					penScratch.Color = penShadow.Color;

					g.FillPolygon(new SolidBrush(color), new PointF[8] {
						new PointF(x + width - space - slope, y),
						new PointF(x + width - space, y + slope),
						new PointF(x + width - space, y + height - space - slope),
						new PointF(x + width - space - slope, y + height - space),
						new PointF(x + slope, y + height - space),
						new PointF(x        , y + height - space - slope),
						new PointF(x        , y + slope),
						new PointF(x + slope, y),
					});
					for (int i = 0; i < density; i++)
					{
						float scratchX = x + (float)rand.Next() / (float)int.MaxValue *
							((float)width - 1.0f - space);
						float scratchY = y + (float)rand.Next() / (float)int.MaxValue *
							((float)height - 1.0f - space);
						g.DrawLine(penScratch, scratchX + 1.0f, scratchY, scratchX, scratchY + 1.0f);
					}
					g.DrawLines(penShadow, new PointF[4] {
						new PointF(x + slope + shadowDelta, y + height - space - shadowDelta),
						new PointF(x + width - space - shadowDelta - slope, y + height - space - shadowDelta),
						new PointF(x + width - space - shadowDelta, y + height - space - shadowDelta - slope),
						new PointF(x + width - space - shadowDelta, y + slope + shadowDelta),
					});
					g.DrawLines(penHighlight, new PointF[4] {
						new PointF(x + shadowDelta, y + height - space - slope - shadowDelta),
						new PointF(x + shadowDelta, y + shadowDelta + slope),
						new PointF(x + shadowDelta + slope, y + shadowDelta),
						new PointF(x + width - space - slope - shadowDelta, y + shadowDelta),
					});
				}
			}
		}

		private void CreateEdgesOrJags(Graphics g)
		{
			Pen penHighlight = new Pen(Color.White, (float)numericUpDownPen1.Value);
			Pen penShadow    = new Pen(Color.Black, (float)numericUpDownPen2.Value);

			int density            = Math.Max(0, (int)numericUpDownDensity.Value);
			float height           = Math.Abs((float)numericUpDownSize.Value);
			float root             = Math.Abs((float)numericUpDownRoot.Value);
			float paddingHighlight = penHighlight.Width * 0.5f;
			float paddingShadow    = penShadow.Width    * 0.5f;
			float slope            = (float)numericUpDownSlope.Value;

			Color color = GetColor(false);
			int brightnessHighlight = (int)numericUpDownBrightness.Value;
			penHighlight.Color = Color.FromArgb(
				Math.Min(255, color.R + brightnessHighlight),
				Math.Min(255, color.G + brightnessHighlight),
				Math.Min(255, color.B + brightnessHighlight));
			penShadow.Color = Color.FromArgb(
				Math.Max(0, color.R - brightnessHighlight),
				Math.Max(0, color.G - brightnessHighlight),
				Math.Max(0, color.B - brightnessHighlight));

			PointF[] pointsHighlight = new PointF[density + 6];
			pointsHighlight[3] = new PointF(paddingHighlight + slope, rand.Next(bitmap.Height));
			pointsHighlight[1] = new PointF(paddingHighlight,
				Math.Max(pointsHighlight[3].Y, bitmap.Height - paddingShadow - slope));
			pointsHighlight[2] = new PointF(paddingHighlight,
				Math.Min(pointsHighlight[1].Y, pointsHighlight[3].Y + slope));
			pointsHighlight[0] = new PointF(paddingHighlight + slope, bitmap.Height - paddingShadow);
			pointsHighlight[pointsHighlight.Length - 2] =
				new PointF(bitmap.Width - paddingShadow - slope, rand.Next(bitmap.Height));
			pointsHighlight[pointsHighlight.Length - 1] =
				new PointF(bitmap.Width - paddingShadow,
				Math.Min(bitmap.Height - paddingShadow, pointsHighlight[pointsHighlight.Length - 2].Y + slope));
			CalculateEdgePoints(ref pointsHighlight, 3, pointsHighlight.Length - 2, height, root);

			PointF[] points = new PointF[pointsHighlight.Length + 3];
			pointsHighlight.CopyTo(points, 0);
			points[points.Length - 3] = new PointF(bitmap.Width - paddingShadow,
				Math.Max(points[points.Length - 4].Y, bitmap.Height - paddingShadow - slope));
			points[points.Length - 2] = new PointF(bitmap.Width - paddingShadow - slope,
				Math.Max(points[points.Length - 3].Y, bitmap.Height - paddingShadow));
			points[points.Length - 1] = new PointF(paddingHighlight + slope, bitmap.Height - paddingShadow);

			PointF[] pointsShadow = new PointF[4];
			pointsShadow[0] = new PointF(paddingHighlight + slope, bitmap.Height - paddingShadow);
			pointsShadow[1] = new PointF(bitmap.Width - paddingShadow - slope, bitmap.Height - paddingShadow);
			pointsShadow[3] = new PointF(bitmap.Width - paddingShadow, pointsHighlight[pointsHighlight.Length - 1].Y);
			pointsShadow[2] = new PointF(bitmap.Width - paddingShadow,
				Math.Max(pointsShadow[3].Y, bitmap.Height - paddingShadow - slope));

			g.FillPolygon(new SolidBrush(color), points);
			g.DrawLines(penShadow, pointsShadow);
			g.DrawLines(penHighlight, pointsHighlight);
		}

		private void CalculateEdgePoints(ref PointF[] points, int indexLeft, int indexRight,
			float height, float root)
		{
			if (indexRight - indexLeft < 2)
				return;

			int indexCurrent = (int)Math.Round((indexLeft + indexRight) / 2.0);
			points[indexCurrent].X =
				(float)Math.Round((points[indexLeft].X + points[indexRight].X) * 0.5f);
			float rndMax = (float)(indexRight - indexLeft) / (float)points.Length;
			float rnd = (rand.Next() / (float)int.MaxValue * height - height * 0.5f) * rndMax;
			rnd = Math.Max(-root, Math.Min(root, rnd));
			points[indexCurrent].Y = Math.Max(0.0f, Math.Min(bitmap.Height - 1.0f,
				(float)Math.Round((points[indexLeft].Y + points[indexRight].Y) * 0.5f) + rnd));
			CalculateEdgePoints(ref points, indexLeft,    indexCurrent, height, root);
			CalculateEdgePoints(ref points, indexCurrent, indexRight,   height, root);
		}
	}
}
